==> hyades.source_1.0.3_20030929_1406.zip: org/eclipse/hyades/loaders/util/GenericLookupService.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v0.5
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v05.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.loaders.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;



/**
 * @author slavescu
 */
public abstract class GenericLookupService extends HashMap implements LookupService {
    //~ Instance fields ----------------------------------------------------------------------------

    protected HashSet types;

    //~ Constructors -------------------------------------------------------------------------------

    public GenericLookupService() {
        super();
        types = new HashSet();
    }

    //~ Methods ------------------------------------------------------------------------------------

    public final Set getSupportedTypes() {
        return types;
    }

    public boolean deregister(HierarchyContext context, Class type, Object key) {
        Map map = getTypeMap(context, type, false);

        if (map == null) {
            return false;
        }

        return map.remove(key) != null;
    }

    public final boolean deregister(HierarchyContext context, Object object) {
        if (object == null) {
            return false;
        }

        return processObject(context, object, true, false);
    }

    public Object locate(HierarchyContext context, Class type, Object key) {
        Map map = getTypeMap(context, type, false);

        if (map == null) {
            return null;
        }

        return (Object) map.get(key);
    }

    public Object locate(HierarchyContext context, Object key) {
        Object o = null;

        try {
            Map contextMap = getContextMap(context, false);

            if (contextMap == null) {
                return null;
            }

            for (Iterator iter = contextMap.values().iterator(); iter.hasNext();) {
                Map typeMap = (Map) iter.next();

                o = typeMap.get(key);

                if (o != null) {
                    return o;
                }
            }
        } catch (Exception e) {
            LoadersUtils.log(e);
        }

        return o;
    }

    public final boolean register(HierarchyContext context, Object object) {
        if (object == null) {
            return false;
        }

        return processObject(context, object, false, true);
    }

    public boolean deregister(HierarchyContext context, Class type) {
        Map typeMap = getTypeMap(context, type, false);

        if (typeMap != null) {
            typeMap.clear();

            return true;
        }

        return false;
    }

    public boolean deregister(HierarchyContext context) {
        Map contextMap = getContextMap(context, false);

        if (contextMap != null) {
            contextMap.clear();

            return true;
        }

        return false;
    }

    protected Map getContextMap(HierarchyContext context, boolean createMap) {
        if (context == null) {
            return this;
        }

        Map contextMap = null;

        try {
            contextMap = (Map) get(context.getContextURI());

            if ((contextMap == null) && createMap) {
                contextMap = new HashMap();
                put(context.getContextURI(), contextMap);
            }
        } catch (Exception e) {
        }

        return contextMap;
    }

    protected abstract boolean doProcessObject(Map typeMap, Class theClass, Object object, boolean remove);

    protected final void addOrRemove(Map map, Object key, Object object, boolean remove) {
        if (remove) {
            map.remove(key);
        } else {
            map.put(key, object);
        }
    }

    protected Map getTypeMap(HierarchyContext context, Class type, boolean create) {
        if (!types.contains(type)) {
            return null;
        }

        Map typeMap = null;

        try {
            Map contextMap = getContextMap(context, create);

            if (contextMap != null) {
                typeMap = (Map) contextMap.get(type);

                if ((typeMap == null) && create) {
                    typeMap = new HashMap();
                    contextMap.put(type, typeMap);
                }
            }
        } catch (Exception e) {
            LoadersUtils.log(e);
        }

        return typeMap;
    }

    private boolean processObject(HierarchyContext context, Object object, boolean remove, boolean createMap) {
		Map typeMap;
		boolean ret = false;
		Class theClass = object.getClass();
    	if(theClass == GenericLookUpEntry.class)
    	{
    		GenericLookUpEntry entry = (GenericLookUpEntry)object;
			typeMap = getTypeMap(context, entry.getValue().getClass(), createMap);
			if (typeMap == null) {
				return false;
			}
			addOrRemove(typeMap, entry.getKey(),entry.getValue(), remove);
			return true;
    	}
    	else
    	{
			typeMap = getTypeMap(context, theClass, createMap);
			if (typeMap == null) {
				return false;
			}

			try {
				ret = doProcessObject(typeMap,theClass,object, remove);
			} catch (Exception e) {
				LoadersUtils.log(e);
			}
    	}

        return ret;
    }
}

==> hyades.source_1.3.0_20040422_2239.zip: org/eclipse/hyades/loaders/util/GenericLookupService.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v0.5
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v05.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.loaders.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;



/**
 * @author slavescu
 */
public abstract class GenericLookupService extends HashMap implements LookupService {
    //~ Instance fields ----------------------------------------------------------------------------

    protected HashSet types;

    //~ Constructors -------------------------------------------------------------------------------

    public GenericLookupService() {
        super();
        types = new HashSet();
    }

    //~ Methods ------------------------------------------------------------------------------------

    public final Set getSupportedTypes() {
        return types;
    }

    public boolean deregister(HierarchyContext context, Class type, Object key) {
        Map map = getTypeMap(context, type, false);

        if (map == null) {
            return false;
        }

        return map.remove(key) != null;
    }

    public final boolean deregister(HierarchyContext context, Object object) {
        if (object == null) {
            return false;
        }

        return processObject(context, object, true, false);
    }

    public Object locate(HierarchyContext context, Class type, Object key) {
        Map map = getTypeMap(context, type, false);

        if (map == null) {
            return null;
        }

        return (Object) map.get(key);
    }

    public Object locate(HierarchyContext context, Object key) {
        Object o = null;

        try {
            Map contextMap = getContextMap(context, false);

            if (contextMap == null) {
                return null;
            }

            for (Iterator iter = contextMap.values().iterator(); iter.hasNext();) {
                Map typeMap = (Map) iter.next();

                o = typeMap.get(key);

                if (o != null) {
                    return o;
                }
            }
        } catch (Exception e) {
            LoadersUtils.log(e);
        }

        return o;
    }

    public final boolean register(HierarchyContext context, Object object) {
        if (object == null) {
            return false;
        }

        return processObject(context, object, false, true);
    }

    public boolean deregister(HierarchyContext context, Class type) {
        Map typeMap = getTypeMap(context, type, false);

        if (typeMap != null) {
            typeMap.clear();

            return true;
        }

        return false;
    }

    public boolean deregister(HierarchyContext context) {
        Map contextMap = getContextMap(context, false);

        if (contextMap != null) {
            contextMap.clear();

            return true;
        }

        return false;
    }

    protected Map getContextMap(HierarchyContext context, boolean createMap) {
        if (context == null) {
            return this;
        }

        Map contextMap = null;

        try {
            contextMap = (Map) get(context.getContextURI());

            if ((contextMap == null) && createMap) {
                contextMap = new HashMap();
                put(context.getContextURI(), contextMap);
            }
        } catch (Exception e) {
        }

        return contextMap;
    }

    protected abstract boolean doProcessObject(Map typeMap, Class theClass, Object object, boolean remove);

    protected final void addOrRemove(Map map, Object key, Object object, boolean remove) {
        if (remove) {
            map.remove(key);
        } else {
            map.put(key, object);
        }
    }

    protected Map getTypeMap(HierarchyContext context, Class type, boolean create) {
    	Class equivalentType=null;
        if (!types.contains(type)) {
        	if(useAssignableFrom(type))
        	{
        		for (Iterator iter = types.iterator(); iter.hasNext();) {
					Class element = (Class) iter.next();
					if(type.isAssignableFrom(element))
					{
						equivalentType = element;
						types.add(type);
						create = true;
						break;
					}
				}
				if(equivalentType == null)
					return null;
        	}
        	else
        	    return null;
        }

        Map typeMap = null;

        try {
            Map contextMap = getContextMap(context, create);

            if (contextMap != null) {
				typeMap = (Map) contextMap.get(type);

                if ((typeMap == null) && create) {
					if(equivalentType != null)
					{
						typeMap = (Map) contextMap.get(equivalentType);
						if(typeMap==null)
						{
							typeMap = new HashMap();
							contextMap.put(equivalentType, typeMap);
						}
						contextMap.put(type, typeMap);
					}
					else
					{
						typeMap = new HashMap();
						contextMap.put(type, typeMap);
					}
                }
            }
        } catch (Exception e) {
            LoadersUtils.log(e);
        }

        return typeMap;
    }

    /**
	 * @return true if we should use isAssignableFrom for this type (then I will search through all supported types to see if this type can be assignable from one of them)
	 */
	protected boolean useAssignableFrom(Class type) {
		return false;
	}

	private boolean processObject(HierarchyContext context, Object object, boolean remove, boolean createMap) {
		Map typeMap;
		boolean ret = false;
		Class theClass = object.getClass();
    	if(theClass == GenericLookUpEntry.class)
    	{
    		GenericLookUpEntry entry = (GenericLookUpEntry)object;
			typeMap = getTypeMap(context, entry.getValue().getClass(), createMap);
			if (typeMap == null) {
				return false;
			}
			addOrRemove(typeMap, entry.getKey(),entry.getValue(), remove);
			return true;
    	}
    	else
    	{
			typeMap = getTypeMap(context, theClass, createMap);
			if (typeMap == null) {
				return false;
			}

			try {
				ret = doProcessObject(typeMap,theClass,object, remove);
			} catch (Exception e) {
				LoadersUtils.log(e);
			}
    	}

        return ret;
    }
}
